--
-- MotorlessHandTool
--
-- Author: Tobias Fricke
-- Description: Specialization for push handtools to work without actual motor
-- Name: MotorlessHandTool
-- Hide: yes
--
-- Copyright (c) Creative Mesh, 2023

---@class MotorlessHandTool
MotorlessHandTool = {}
MotorlessHandTool.MOD_NAME = g_currentModName

function MotorlessHandTool.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations)
        and SpecializationUtil.hasSpecialization(Enterable, specializations)
end

function MotorlessHandTool.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "getMotorNotAllowedWarning", MotorlessHandTool.getMotorNotAllowedWarning)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "updateMotorProperties", MotorlessHandTool.updateMotorProperties)
end

function MotorlessHandTool.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", MotorlessHandTool)
    SpecializationUtil.registerEventListener(vehicleType, "onUpdateTick", MotorlessHandTool)
end

---Called on load.
function MotorlessHandTool:onLoad(savegame)
    self.spec_motorlessHandTool = self[("spec_%s.motorlessHandTool"):format(MotorlessHandTool.MOD_NAME)]

    local spec_motorized = self.spec_motorized
    spec_motorized.forceSpeedHudDisplay = true
end

---Called on update tick
---@param dt number delta time since last frame
function MotorlessHandTool:onUpdateTick(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
    local spec_motorized = self.spec_motorized

    if not spec_motorized.isMotorStarted then
        if self.getIsControlled ~= nil and self:getIsControlled() and self:getCanMotorRun() then
            self:startMotor()
        end

        if self:getRequiresPower() and self:getCanMotorRun() then
            self:startMotor()
        end
    elseif self.getIsControlled ~= nil and not self:getIsControlled() and self:getStopMotorOnLeave() then
        spec_motorized.motorNotRequiredTimer = spec_motorized.motorNotRequiredTimer + dt

        if spec_motorized.motorNotRequiredTimer > 250 then
            self:stopMotor()
        end

        self:raiseActive()
    end
end

----------------
---Overwrites---
----------------

---Overwritten function: getMotorNotAllowedWarning
---@param superFunc function overwritten function
---@return string|nil warning
function MotorlessHandTool:getMotorNotAllowedWarning(superFunc)
    -- dont render any motor warnings
    return nil
end

---Overwritten function: updateMotorProperties
---@param superFunc function overwritten function
function MotorlessHandTool:updateMotorProperties(superFunc)
    local spec = self.spec_motorized
    if spec.motorizedNode ~= nil then
        superFunc(self)
    end
end
